<?php

namespace App\Service;

use App\Exception\ParamsException as ParamsException;
use App\Exception\SqlException as SqlException;
use App\Exception\CancelException as CancelException;

class OrderService {

    private $orderFactory;
    private $util;
    

    public function __construct (\App\Factory\OrderFactory $orderFactory, 
                                 \App\Util\Util $util) {

        $this->orderFactory  = $orderFactory;
        $this->util          = $util;
    }
    
    public function getDefaultParams($cdfilial) {
        $defaultParams = $this->orderFactory->getDefaultParams($cdfilial);
        $this->validateDefaultParams($defaultParams);
        return $defaultParams;
    }
    
    public function getBranch($token) {
        return $this->orderFactory->getBranch($token);
    }

    private function validateDefaultParams($params) {

        if(empty($params)) 
            throw new ParamsException("Nenhum caixa foi encontrado.");
        
        if(empty($params['CDVENDEDOR'])) 
            throw new ParamsException("Vendedor padrao nao encontrado.");
        
        if(empty($params['CDOPERADOR'])) 
            throw new ParamsException("O vendedor {$params['NMFANVEN']} nao esta associado a um operador valido.");
        
        if(empty($params['CDLOJA'])) 
            throw new ParamsException("Loja nao encontrada para o caixa {$params['CDCAIXA']}.");
        
        if(empty($params['NRMESA'])) 
            throw new ParamsException("Mesa padrão nao encontrada para a loja {$params['CDLOJA']}.");
        
        if(empty($params['CDCLIENTE'])) 
            throw new ParamsException("Cliente padrão nao encontrado para filial {$params['CDFILIAL']}.");
        
        if(empty($params['CDCONSUMIDOR'])) 
            throw new ParamsException("Consumidor padrão nao encontrado.");
    }

    public function saveVendarest($cdfilial, $cdloja, $nrmesa, $cdvendedor, $cdoperador, $cdcliente, $cdconsumidor, $sgestado) {
        $nrvendarest = $this->util->newCode("VENDAREST$cdfilial", 10);
        $vendarest   = $this->orderFactory->saveVendarest($cdfilial, $nrvendarest, $cdloja, $nrmesa, $cdvendedor, $cdoperador, $cdcliente, $cdconsumidor, $sgestado);

        if(empty($vendarest)) 
            throw new SqlException("Houve um erro ao gravar na tabela VENDAREST.");

        return $nrvendarest;
    }

    public function saveComandaven($cdfilial, $nrvendarest, $nrcomandaext, $cdloja, $idstcomanda, $deliveryFee, $nrpedmobile, $consumerData, $dsObsComanda, $consumerIdentifier, $dlvMethod) {
        $nrcomanda  = $this->util->newCode("COMANDAVEN$cdfilial", 10);
        
        $comandaven = $this->orderFactory->saveComandaven($cdfilial,$nrvendarest,$nrcomanda, $nrcomandaext, $cdloja, "DLV_$nrcomanda", $idstcomanda, $deliveryFee, $consumerData, $dsObsComanda, $nrpedmobile, $consumerIdentifier, $dlvMethod);
        
        if(empty($comandaven)) 
            throw new SqlException("Houve um erro ao gravar na tabela COMANDAVEN.");
        
        return $nrcomanda;
    }
    
    public function validateDuplication($defaultBranch, $resourceId) {
        return $this->orderFactory->validateDuplication($defaultBranch, $resourceId);
    }
    
    public function cancelOrder($orderId) {
        try {
            $orderData = $this->orderFactory->getOrderDataById($orderId);

            if(empty($orderData))
                throw new CancelException("O pedido a ser cancelado não foi integrado.");
                
            $orderStatus = $orderData['IDSTCOMANDA'];
            
            if($orderStatus == 'P' || $orderStatus == '5' || $orderStatus == '6' || $orderStatus == '4' || $orderStatus == 'X')
                throw new CancelException("O pedido $orderId não pode ser cancelado. Motivo: O pedido encontra-se com o seguinte status: " . $this->getStatusString($orderStatus));
                
            $this->orderFactory->cancelOrder($orderData);
        } catch(\Exception $e) {
            throw new CancelException($e->getMessage());
        }
    }
    
    public function getOrderDetailsById($resourceId){
        try{
            return $this->orderFactory->getOrderDetailsById($resourceId);
        } catch(\Exception $e){
            throw new CancelException($e->getMessage());
        }
    }
	
    public function validateOrder($cdFilial, $nrVendaRest){
        $this->orderFactory->validateOrder($cdFilial, $nrVendaRest);
    }
    
    public function getDeliveryMethodParams($cdFilial) {
        $deliveryParams = $this->orderFactory->getDeliveryMethodParams($cdFilial);

        $filiToken = explode('|', $deliveryParams['IDTOKEN']);
        if(is_array($filiToken) && count($filiToken) == 2){
            $deliveryParams['IDTOKENINTGERAL'] = $filiToken[1];
        }

        return $deliveryParams;
    }
    
    public function getStatusString($id) {
        return $this->statusList[$id];
    }
    
    private $statusList = array(
        1   => 'Em produção',
        2   => 'Em produção',
        3   => 'Em produção',
        4   => 'Cancelado',
        5   => 'Expedido',
        6   => 'Não Pago',
        'X' => 'Finalizado',
        'P' => 'Impresso',
        'A' => 'Agendado',
        'N' => 'Não Sincronizado'
    );
}
